@@ -3,21 +3,27 @@ from urllib.parse import urljoin
 from tornado import web
 from tornado.ioloop import IOLoop
 
-from .components.nvml import gpu_doc  # noqa: 1708
-from .components.nvml import NVML_ENABLED, gpu_memory_doc, gpu_utilization_doc
-from .components.scheduler import (
+from distributed.dashboard.components.nvml import gpu_doc  # noqa: 1708
+from distributed.dashboard.components.nvml import (
+    NVML_ENABLED,
+    gpu_memory_doc,
+    gpu_utilization_doc,
+)
+from distributed.dashboard.components.scheduler import (
     AggregateAction,
     BandwidthTypes,
     BandwidthWorkers,
     ClusterMemory,
     ComputePerKey,
     CurrentLoad,
+    EventLoop,
     MemoryByKey,
     Occupancy,
     SystemMonitor,
     SystemTimeseries,
     TaskGraph,
     TaskGroupGraph,
+    TaskGroupProgress,
     TaskProgress,
     TaskStream,
     WorkerNetworkBandwidth,
@@ -25,6 +31,7 @@ from .components.scheduler import (
     WorkerTable,
     events_doc,
     graph_doc,
+    hardware_doc,
     individual_doc,
     individual_profile_doc,
     individual_profile_server_doc,
@@ -37,8 +44,8 @@ from .components.scheduler import (
     tg_graph_doc,
     workers_doc,
 )
-from .core import BokehApplication
-from .worker import counters_doc
+from distributed.dashboard.core import BokehApplication
+from distributed.dashboard.worker import counters_doc
 
 applications = {
     "/system": systemmonitor_doc,
@@ -51,6 +58,7 @@ applications = {
     "/profile": profile_doc,
     "/profile-server": profile_server_doc,
     "/graph": graph_doc,
+    "/hardware": hardware_doc,
     "/groups": tg_graph_doc,
     "/gpu": gpu_doc,
     "/individual-task-stream": individual_doc(
@@ -59,6 +67,7 @@ applications = {
     "/individual-progress": individual_doc(TaskProgress, 100, height=160),
     "/individual-graph": individual_doc(TaskGraph, 200),
     "/individual-groups": individual_doc(TaskGroupGraph, 200),
+    "/individual-group-progress": individual_doc(TaskGroupProgress, 200),
     "/individual-workers-memory": individual_doc(WorkersMemory, 100),
     "/individual-cluster-memory": individual_doc(ClusterMemory, 100),
     "/individual-cpu": individual_doc(CurrentLoad, 100, fig_attr="cpu_figure"),
@@ -91,6 +100,7 @@ applications = {
     "/individual-compute-time-per-key": individual_doc(ComputePerKey, 500),
     "/individual-aggregate-time-per-action": individual_doc(AggregateAction, 500),
     "/individual-scheduler-system": individual_doc(SystemMonitor, 500),
+    "/individual-event-loop": individual_doc(EventLoop, 500),
     "/individual-profile": individual_profile_doc,
     "/individual-profile-server": individual_profile_server_doc,
     "/individual-gpu-memory": gpu_memory_doc,
@@ -98,7 +108,7 @@ applications = {
 }
 
 
-template_variables = {
+template_variables: dict = {
     "pages": [
         "status",
         "workers",
@@ -109,8 +119,22 @@ template_variables = {
         "groups",
         "info",
     ],
-    "plots": [x.replace("/", "") for x in applications if "individual" in x],
+    "plots": [
+        {
+            "url": x.strip("/"),
+            "name": " ".join(x.strip("/").split("-")[1:])
+            .title()
+            .replace("Cpu", "CPU")
+            .replace("Gpu", "GPU"),
+        }
+        for x in applications
+        if "individual" in x
+    ]
+    + [{"url": "hardware", "name": "Hardware"}],
 }
+template_variables["plots"] = sorted(
+    template_variables["plots"], key=lambda d: d["name"]
+)
 
 if NVML_ENABLED:
     template_variables["pages"].insert(4, "gpu")
