@@ -1,10 +1,12 @@
 from collections.abc import MutableMapping
 
-from .utils import log_errors, tokey
+from dask.utils import stringify
+
+from distributed.utils import log_errors
 
 
 class PublishExtension:
-    """ An extension for the scheduler to manage collections
+    """An extension for the scheduler to manage collections
 
     *  publish_list
     *  publish_put
@@ -24,30 +26,27 @@ class PublishExtension:
         }
 
         self.scheduler.handlers.update(handlers)
-        self.scheduler.extensions["publish"] = self
 
-    def put(
-        self, comm=None, keys=None, data=None, name=None, override=False, client=None
-    ):
+    def put(self, keys=None, data=None, name=None, override=False, client=None):
         with log_errors():
             if not override and name in self.datasets:
                 raise KeyError("Dataset %s already exists" % name)
-            self.scheduler.client_desires_keys(keys, "published-%s" % tokey(name))
+            self.scheduler.client_desires_keys(keys, f"published-{stringify(name)}")
             self.datasets[name] = {"data": data, "keys": keys}
             return {"status": "OK", "name": name}
 
-    def delete(self, comm=None, name=None):
+    def delete(self, name=None):
         with log_errors():
             out = self.datasets.pop(name, {"keys": []})
             self.scheduler.client_releases_keys(
-                out["keys"], "published-%s" % tokey(name)
+                out["keys"], f"published-{stringify(name)}"
             )
 
     def list(self, *args):
         with log_errors():
             return list(sorted(self.datasets.keys(), key=str))
 
-    def get(self, stream, name=None, client=None):
+    def get(self, name=None, client=None):
         with log_errors():
             return self.datasets.get(name, None)
 
@@ -94,8 +93,7 @@ class Datasets(MutableMapping):
                 "Can't invoke iter() or 'for' on client.datasets when client is "
                 "asynchronous; use 'async for' instead"
             )
-        for key in self._client.list_datasets():
-            yield key
+        yield from self._client.list_datasets()
 
     def __aiter__(self):
         if not self._client.asynchronous:
