@@ -4,9 +4,9 @@ import asyncio
 import logging
 import uuid
 from collections import defaultdict, deque
+from collections.abc import Container
 from math import log2
 from time import time
-from typing import Container
 
 from tlz import topk
 from tornado.ioloop import PeriodicCallback
@@ -14,10 +14,10 @@ from tornado.ioloop import PeriodicCallback
 import dask
 from dask.utils import parse_timedelta
 
-from .comm.addressing import get_address_host
-from .core import CommClosedError, Status
-from .diagnostics.plugin import SchedulerPlugin
-from .utils import log_errors, recursive_to_dict
+from distributed.comm.addressing import get_address_host
+from distributed.core import CommClosedError, Status
+from distributed.diagnostics.plugin import SchedulerPlugin
+from distributed.utils import log_errors, recursive_to_dict
 
 # Stealing requires multiple network bounces and if successful also task
 # submission which may include code serialization. Therefore, be very
@@ -71,7 +71,6 @@ class WorkStealing(SchedulerPlugin):
         )
         # `callback_time` is in milliseconds
         self.scheduler.add_plugin(self)
-        self.scheduler.extensions["stealing"] = self
         self.scheduler.events["stealing"] = deque(maxlen=100000)
         self.count = 0
         # { task state: <stealing info dict> }
@@ -108,31 +107,16 @@ class WorkStealing(SchedulerPlugin):
             pc.stop()
         await self._in_flight_event.wait()
 
-    def _to_dict(self, *, exclude: Container[str] = ()) -> dict:
-        """
-        A very verbose dictionary representation for debugging purposes.
-        Not type stable and not inteded for roundtrips.
-
-        Parameters
-        ----------
-        comm:
-        exclude:
-            A list of attributes which must not be present in the output.
+    def _to_dict_no_nest(self, *, exclude: Container[str] = ()) -> dict:
+        """Dictionary representation for debugging purposes.
+        Not type stable and not intended for roundtrips.
 
         See also
         --------
         Client.dump_cluster_state
+        distributed.utils.recursive_to_dict
         """
-        return recursive_to_dict(
-            {
-                "stealable_all": self.stealable_all,
-                "stealable": self.stealable,
-                "key_stealable": self.key_stealable,
-                "in_flight": self.in_flight,
-                "in_flight_occupancy": self.in_flight_occupancy,
-            },
-            exclude=exclude,
-        )
+        return recursive_to_dict(self, exclude=exclude, members=True)
 
     def log(self, msg):
         return self.scheduler.log_event("stealing", msg)
@@ -273,7 +257,7 @@ class WorkStealing(SchedulerPlugin):
         except CommClosedError:
             logger.info("Worker comm %r closed while stealing: %r", victim, ts)
             return "comm-closed"
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             logger.exception(e)
             if LOG_PDB:
                 import pdb
@@ -353,7 +337,7 @@ class WorkStealing(SchedulerPlugin):
                 self.log(("confirm", *_log_msg))
             else:
                 raise ValueError(f"Unexpected task state: {state}")
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             logger.exception(e)
             if LOG_PDB:
                 import pdb
@@ -430,10 +414,7 @@ class WorkStealing(SchedulerPlugin):
                         if not idle:
                             break
 
-                        if _has_restrictions(ts):
-                            thieves = [ws for ws in idle if _can_steal(ws, ts, sat)]
-                        else:
-                            thieves = idle
+                        thieves = _potential_thieves_for(ts, idle)
                         if not thieves:
                             break
                         thief = thieves[i % len(thieves)]
@@ -466,10 +447,7 @@ class WorkStealing(SchedulerPlugin):
                             continue
 
                         i += 1
-                        if _has_restrictions(ts):
-                            thieves = [ws for ws in idle if _can_steal(ws, ts, sat)]
-                        else:
-                            thieves = idle
+                        thieves = _potential_thieves_for(ts, idle)
                         if not thieves:
                             continue
                         thief = thieves[i % len(thieves)]
@@ -507,18 +485,16 @@ class WorkStealing(SchedulerPlugin):
         return out
 
 
-def _has_restrictions(ts):
-    """Determine whether the given task has restrictions and whether these
-    restrictions are strict.
-    """
-    return not ts.loose_restrictions and (
-        ts.host_restrictions or ts.worker_restrictions or ts.resource_restrictions
-    )
+def _potential_thieves_for(ts, idle):
+    """Return the list of workers from ``idle`` that could steal ``ts``."""
+    if _has_restrictions(ts):
+        return [ws for ws in idle if _can_steal(ws, ts)]
+    else:
+        return idle
 
 
-def _can_steal(thief, ts, victim):
-    """Determine whether worker ``thief`` can steal task ``ts`` from worker
-    ``victim``.
+def _can_steal(thief, ts):
+    """Determine whether worker ``thief`` can steal task ``ts``.
 
     Assumes that `ts` has some restrictions.
     """
@@ -544,4 +520,13 @@ def _can_steal(thief, ts, victim):
     return True
 
 
+def _has_restrictions(ts):
+    """Determine whether the given task has restrictions and whether these
+    restrictions are strict.
+    """
+    return not ts.loose_restrictions and (
+        ts.host_restrictions or ts.worker_restrictions or ts.resource_restrictions
+    )
+
+
 fast_tasks = {"split-shuffle"}
